local DELAY_BASE = 100
local FILTERS = {"All", "Armed Guard", "Sniper"}
setmetatable(FILTERS, {__index = function (t, k) Game.DebugOut("[WARNING] Attempted to get invalid filter state " .. tostring(k) .. "!") return "All" end})

local ticksLeft = -1
local currentState = 0

local function isActive()
  local trigger = this.Triggered
  if trigger then
    return trigger == true or trigger == 2 or trigger == 4 or trigger == 5
  else
    return false
  end
end

function Create()
  this.SubType = 0
  this.Range = 12
  this.Filter = 1
  this.AlwaysOn = false
  currentState = 0
end

local function updateRangeDisplay()
  this.SetInterfaceCaption("RangeDisplay", tostring(this.Range));
end

function AddRangeClicked()
  this.Range = this.Range + 1
  updateRangeDisplay()
end

function SubtractRangeClicked()
  this.Range = math.max(this.Range - 1, 1)
  updateRangeDisplay()
end

function AlwaysOnClicked()
  this.AlwaysOn = not this.AlwaysOn
  this.SetInterfaceCaption("AlwaysOn", tostring(this.AlwaysOn))
  currentState = (this.AlwaysOn or isActive()) and 1 or 0
  this.SubType = currentState
end

function FilterClicked()
  local nextFilter = this.Filter + 1
  if nextFilter >= 4 then
    nextFilter = 1
  end
  this.SetInterfaceCaption("Filter", FILTERS[nextFilter])
  this.Filter = nextFilter
end

local function getNumPrisoners()
  local prisonerTable = World.NumPrisoners
  local total = 0
  for k, v in pairs(prisonerTable) do
    total = total + v
  end
  return total
end

--[[
  Here's the super ugly hack used for sniper "freefire":
  
  1. Track the sniper's target
  2. Use AimTimer to determine when the sniper shoots
  3. Once the sniper shoots set prisoner damage to .99 and give the wounded status effect
  4. The really small wounded value will kill the prisoner almost instantly and will count towards deaths
  
  Sadly, the WarningShots property won't work because it seems the game deals sniper damage after
  deciding if the prisoner surrenders. In my testing this pretty much meant that regular prisoners would
  surrender and fearless ones would get shot. Also, with global prison freefire off sniper shots always only
  deal enough damage to knock prisoners unconscious. I do set the property anyway just to not mess up other things.
  
  Also, custom properties set on vanilla objects start with FFC_ to avoid possible conflicts and to make it clear this mod
  created them.
]]

local function doUpdate()
  print(World.TimeWarpFactor)
  ticksLeft = ticksLeft - 1
  if ticksLeft <= 0 then
    if this.AlwaysOn or isActive() then
      if currentState == 0 then
        this.SubType = 1
        currentState = 1
      end
      local numGuards = 0
      if this.Filter == 1 or this.Filter == 2 then
        for entity, distance in pairs(this.GetNearbyObjects("ArmedGuard", this.Range)) do
          if not World.LethalForce then
            entity.WeaponDrawn = 10
          end
          numGuards = numGuards + 1
        end
      end
      if this.Filter == 1 or this.Filter == 3 then
        for sniper, d in pairs(this.GetNearbyObjects("Sniper", this.Range)) do
          if not World.LethalForce then
            sniper.WarningShots = 3
            if (sniper.TargetPrisoner.i ~= -1 and sniper.TargetPrisoner.u ~= -1) or Object.GetProperty(sniper, "FFC_DeathTime") ~= -1 then
              if not Object.GetProperty(sniper, "FFC_TargetIdI") or Object.GetProperty(sniper, "FFC_TargetIdI") == -1 then
                Object.SetProperty(sniper, "FFC_TargetIdI", sniper.TargetPrisoner.i)
                Object.SetProperty(sniper, "FFC_TargetIdU", sniper.TargetPrisoner.u)
                Object.SetProperty(sniper, "FFC_DeathTime", Game.Time() + sniper.AimTimer)
              elseif Game.Time() >= Object.GetProperty(sniper, "FFC_DeathTime") then
                for prisoner, dist in pairs(this.GetNearbyObjects("Prisoner", 85)) do
                  if prisoner.Id.i == Object.GetProperty(sniper, "FFC_TargetIdI") and prisoner.Id.u == Object.GetProperty(sniper, "FFC_TargetIdU") then
                    if prisoner.Damage < 0.7 or (prisoner.Attacker and prisoner.Attacker.i == sniper.Id.i and prisoner.Attacker.u == sniper.Id.u) then
                      prisoner.Damage = 0.99
                      prisoner.StatusEffects.wounded = 0.001
                    end
                    break
                  end
                end
                Object.SetProperty(sniper, "FFC_TargetIdI", -1)
                Object.SetProperty(sniper, "FFC_TargetIdU", -1)
                Object.SetProperty(sniper, "FFC_DeathTime", -1)
              end
            else
              Object.SetProperty(sniper, "FFC_TargetIdI", -1)
              Object.SetProperty(sniper, "FFC_TargetIdU", -1)
              Object.SetProperty(sniper, "FFC_DeathTime", -1)
            end
          end
          numGuards = numGuards + 1
        end
      end
      --have to keep updating this tooltip for # of guards in range
      this.Tooltip = {"tooltip_active_range", tostring(numGuards), "X"}
    else
      if currentState == 1 then
        if not World.LethalForce and (this.Filter == 1 or this.Filter == 2) then
          for entity, distance in pairs(this.GetNearbyObjects("ArmedGuard", this.Range)) do
            entity.WeaponDrawn = 0
          end
          if this.Filter == 1 or this.Filter == 3 then
            for sniper, d in pairs(this.GetNearbyObjects("Sniper", this.Range)) do
              sniper.WarningShots = 0
              Object.SetProperty(sniper, "FFC_TargetIdI", -1)
              Object.SetProperty(sniper, "FFC_TargetIdU", -1)
              Object.SetProperty(sniper, "FFC_DeathTime", -1)
            end
          end
        end
        this.SubType = 0
        currentState = 0
        this.Tooltip = "tooltip_inactive"
      end
    end
    ticksLeft = math.max(1, DELAY_BASE - getNumPrisoners() / 5)
  end
end

function Update()
  this.Range = this.Range or 12
  this.Filter = this.Filter or 1
  this.AlwaysOn = this.AlwaysOn or false
  currentState = this.SubType
  this.AddInterfaceComponent("AlwaysOnCaption", "Caption", "interface_alwayson")
  this.AddInterfaceComponent("AlwaysOn", "Button", tostring(this.AlwaysOn))
  this.AddInterfaceComponent("Empty1", "Caption", "")
  this.AddInterfaceComponent("FilterCaption", "Caption", "interface_filter")
  this.AddInterfaceComponent("Filter", "Button", FILTERS[this.Filter])
  this.AddInterfaceComponent("Empty2", "Caption", "")
  this.AddInterfaceComponent("Range", "Caption", "interface_range")
  this.AddInterfaceComponent("AddRange", "Button", "interface_range_add")
  this.AddInterfaceComponent("RangeDisplay", "Caption", tostring(this.Range))
  this.AddInterfaceComponent("SubtractRange", "Button", "interface_range_subtract")
  
  Update = doUpdate
end
