local DELAY_BASE = 120

local GUARD_LEAVE_THRESHOLD = 10

local ticksLeft = -1
local owner = nil
local currentState = 0

local function isActive()
  local trigger = this.Triggered
  if trigger then
    return trigger == true or trigger == 2 or trigger == 4 or trigger == 5
  else
    return false
  end
end

local function findArmedGuard(idI, idU)
  for entity, dist in pairs(this.GetNearbyObjects("ArmedGuard", 9999)) do
    if entity.Id.i == idI and entity.Id.u == idU then
      return entity
    end
  end
  return nil
end

local function isPatrolCurrentlyNear(entity)
  return math.floor(entity.Patrol["LastPos.x"]) == math.floor(this.Pos.x) and math.floor(entity.Patrol["LastPos.y"]) == math.floor(this.Pos.y)
end

local function getNumPrisoners()
  local prisonerTable = World.NumPrisoners
  local total = 0
  for k, v in pairs(prisonerTable) do
    total = total + v
  end
  return total
end

local function distanceSquared(entity)
  return math.pow(entity.Pos.x - this.Pos.x, 2) + math.pow(entity.Pos.y - this.Pos.y, 2)
end

function AlwaysOnClicked()
  this.AlwaysOn = not this.AlwaysOn
  this.SetInterfaceCaption("AlwaysOn", tostring(this.AlwaysOn))
  currentState = (this.AlwaysOn or isActive()) and 1 or 0
  this.SubType = currentState
end

function Create()
  this.OwnerIdI = -1
  this.OwnerIdU = -1
  this.LastPatrolX = -1
  this.LastPatrolY = -1
  this.MoveFails = 0
  this.AlwaysOn = false
  this.SubType = 0
  currentState = 0
end

local function doUpdate()
  ticksLeft = ticksLeft - 1
  if ticksLeft <= 0 then
    if this.AlwaysOn or isActive() then
      if currentState == 0 then
        this.SubType = 1
        currentState = 1
      end
      if this.OwnerIdI == -1 or this.OwnerIdU == -1 then
        for entity, dist in pairs(this.GetNearbyObjects("ArmedGuard", 0.75)) do
          if entity.Damage < 0.7 and entity.Patrol and isPatrolCurrentlyNear(entity) then
            owner = entity
            this.OwnerIdI = owner.Id.i
            this.OwnerIdU = owner.Id.u
            break
          end
        end
      elseif this.OwnerIdI ~= -1 or this.OwnerIdU ~= -1 then
        if this.LastPatrolX == owner.Patrol["LastPos.x"] and this.LastPatrolY == owner.Patrol["LastPos.y"] then
          this.MoveFails = this.MoveFails + 1
          if this.MoveFails >= GUARD_LEAVE_THRESHOLD then
            --check for armed guard standing in place or fighting, they may still be patrolling on the pad
            if distanceSquared(owner) <= 1 or (owner.TargetObject.i ~= -1 and owner.TargetObject.u ~= -1) then
              this.MoveFails = 0
            else
              owner = nil
            end
          end
        else 
          this.LastPatrolX = owner.Patrol["LastPos.x"]
          this.LastPatrolY = owner.Patrol["LastPos.y"]
          this.MoveFails = 0
        end
        if not owner or owner.Damage >= 0.7 or not owner.Patrol then
          owner = nil
          this.OwnerIdI = -1
          this.OwnerIdU = -1
          this.LastPatrolX = -1
          this.LastPatrolY = -1
        else
          if not World.LethalForce then
            owner.WeaponDrawn = 10
          end
        end
      end
      this.Tooltip = {"tooltip_active_patrol", tostring(owner ~= nil), "X"}
    else
      if currentState == 1 then
        this.SubType = 0
        currentState = 0
        if owner then
          owner.WeaponDrawn = 0
          owner = nil
          this.OwnerIdI = -1
          this.OwnerIdU = -1
          this.LastPatrolX = -1
          this.LastPatrolY = -1
        end
        this.Tooltip = "tooltip_inactive"
      end
    end
    ticksLeft = math.max(1, DELAY_BASE - getNumPrisoners() / 5)
  end
end

function Update()
  this.AlwaysOn = this.AlwaysOn or false
  currentState = this.SubType
  this.OwnerIdI = this.OwnerIdI or -1
  this.OwnerIdU = this.OwnerIdU or -1
  if this.OwnerIdI ~= -1 and this.OwnerIdU ~= -1 then
    owner = findArmedGuard(this.OwnerIdI, this.OwnerIdU)
    if owner then
      this.LastPatrolX = owner.Patrol["LastPos.x"]
      this.LastPatrolY = owner.Patrol["LastPos.y"]
    else
      this.LastPatrolX = -1
      this.LastPatrolY = -1
    end
  else
    this.LastPatrolX = -1
    this.LastPatrolY = -1
  end
  this.MoveFails = this.MoveFails or 0
  this.AddInterfaceComponent("AlwaysOnCaption", "Caption", "interface_alwayson")
  this.AddInterfaceComponent("AlwaysOn", "Button", tostring(this.AlwaysOn))
  Update = doUpdate
end